
import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'shield_config.dart';

class Shield {

  static const MethodChannel _channel = MethodChannel('plugin_shieldfraud');
  static ShieldCallback? _shieldCallback;
  static initShield(ShieldConfig config) async  {
    try {
      if (config.shieldCallback != null) {
        _channel.setMethodCallHandler(_methodHandler);
        _shieldCallback = config.shieldCallback;
      }
       await _channel.invokeMethod("initShieldFraud", {
        "siteID": config.siteID,
        "key": config.key,
        "registerCallback": config.shieldCallback != null,
        "enableBackgroundListener": config.enableBackgroundListener,
        "enableShieldProcess": config.enableShieldProcess,
        "enableMocking": config.enableMocking,
        "partnerId": config.partnerId,
        "environment": config.environment.toString(),
        "logLevel": config.logLevel.toString()
      });
    } on PlatformException catch(_){
      //something went wrong during initialization. nothing we can do
    }
    return;
  }

  static Future<String> get sessionId async {
    String sessionID = "";
    try {
      sessionID = await _channel.invokeMethod('getSessionID');
    } on PlatformException catch (_){
      //something went wrong getting sessionID
    }
    return sessionID;
  }
  static ShieldError? latestError;
  static Future<Map<String, dynamic>?> get latestDeviceResult async {
    var result = "";
    try {
      result = await _channel
          .invokeMethod('getDeviceResult');
      latestError = null;
      return json.decode(result);
    } on PlatformException catch(e){
      latestError = ShieldError(int.parse(e.code), e.message??"Unknown error");
    }
    return null;
  }

  static Future<bool> sendAttributes(String screenName, Map<String, String> data) async {
    var status = false;
    try {
       status = await _channel.invokeMethod("sendAttributes", {
        "screenName": screenName,
        "attributes": data
      });
    } on PlatformException catch(e) {}
      return status;
  }

  static Future<void> _methodHandler(MethodCall call) async{
    try {
      switch (call.method) {
        case "setDeviceResult" :
          {
            _shieldCallback?.onSuccess(json.decode(call.arguments));
            break;
          }
        case "setDeviceResultError" :
          {
            ShieldError shieldError = ShieldError(call.arguments["code"]??0, call.arguments["message"]??"Unknown error");
            _shieldCallback?.onError(shieldError);
            break;
          }
      }
    } on PlatformException catch(e){
      ShieldError shieldError = ShieldError(int.parse(e.code), e.message??"Unknown error");
      _shieldCallback?.onError(shieldError);
    }
  }

}

